function [CRLB,mdat] = CRLB_PHA_with_pileup(spectran,eta, varargin)
% function [CRLB,mdat] = CRLB_PHA_with_pileup(spectran,Tintegrate,deadtime, varargin)
%  Computes Cramer-Rao lower bound (CRLB) for log PHA data with pileup 
%  directly from general formula for Fisher Info matrix
%  for general Gaussian case (Eq. 3.31 of Kay). See Onenote with keyword CRLB for formula
% Also assumes counting detectors with pileup using zero pulse width approximation
%    note that for non-zero eta stats are not Poisson
%  inputs: 
%    spectran:  struct of spectrum transmitted through object. Required fields
%        specnum: the photon number spectrum. Integral is total counts during measurement period
%        egys: (keV) the x-ray energies for each entry of specnum
%        mus: matrix of samples of the basis set functions at egys. Each function is separate column
%        idx_threshold: indexes into egys and specnum arrays for edges of bins.
%           this indirectly specifies nbins = numel(idx_threshold) + 1
%     eta: counts per dead time
%     dA_offset: (optional = 0.001*ones(1,nbasis))  the A-vector step offsets for derivative calculation
%  outputs:
%    CRLB: the CRLB with pileup
%    mdat: debugging info struct. Has workspace at end of executing the function
%  REA 10/1/10 - 4/Feb/2015 10:04
% COPYRIGHT:   Aprend Technology, 2011-2015. All rights reserved.   

   % get nbasis parameter for use by local funcs
[spectran.negys,spectran.nbasis] = size(spectran.mus);
dA_offset = 0.001*ones(1,spectran.nbasis);  % the step offsets for derivative calculation

nreqargs = 2;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'da_offset';         dA_offset=varargin{i+1};   assert(numel(dA_offset)==spectran.nbasis);  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

dAs = diag(dA_offset);

F2 = zeros(spectran.nbasis); % 2nd term of Fisher information matrix
[dCdA,RL] = dCdALOC(spectran, eta,dAs);
RLinv = inv(RL);
   % first accumulate the derivative of covariance terms
for i = 1:spectran.nbasis
   for j= 1:spectran.nbasis
     F2(i,j) = 0.5*trace(RLinv*dCdA(:,:,i)*RLinv*dCdA(:,:,j));
   end
end

M = MLOC(spectran, eta,dAs);
F1 = M'*RLinv*M;
F = F1 + F2;
CRLB = inv(F);
if nargout >1 % save workspace
    CRLB_const_cov = inv(F1);
    mdat = GetWorkspace;
end

%------------------------------ LOCAL FUNCTIONS ----------------------------------

    % ---- derivative of covariance vs. A vector components
function [dRdA,cv0] = dCdALOC(specdat, eta,dAs)
nbins = numel(specdat.idx_threshold) + 1;
dRdA = zeros(nbins,nbins,specdat.nbasis); % nbasis planes, one for each A-vector component
cv0 = covLOC(specdat, eta); % covariance at operating point
for kdim = 1:specdat.nbasis 
    sp2 = specdat;
    sp2.specnum = exp(-specdat.mus*dAs(kdim,:)').*specdat.specnum; % transmitted spectrum -- note transpose
    cv = covLOC(sp2, eta);
    dRdA(:,:,kdim) = (cv - cv0)/dAs(kdim,kdim);
end

     %------------
function [cov_logNK,nbins] = covLOC(specdat, eta)

sp_w_pileup = SpectrumWithPileup(specdat,eta);
    % nbins PHA
[fracs,nbins] = BinFracsFromIndexes(sp_w_pileup, specdat.idx_threshold);
number_photons = trapz(specdat.egys,specdat.specnum); % 
Nrec_bar = (number_photons/(1 + eta)); % mean recorded counts
Nrec_var = number_photons/(1 + eta)^3; % variance of recorded counts
dnon_poiss = Nrec_var - Nrec_bar; % this would be zero for Poisson
nrec_bins = Nrec_bar*fracs; % nbars for each bin
nrec_var_bins = Nrec_bar*fracs + dnon_poiss*(fracs.^2);
cov_logNK = diag(nrec_var_bins./(nrec_bins.^2));
    % compute the off diagonal elements of covariance
    % covariance is symmetric so only do one triangle
[r,c] = find(triu(true(size(cov_logNK)),1)); % indices to upper triangle
for k = 1:numel(r)
    rk = r(k);
    ck = c(k);
    cv = fracs(rk)*fracs(ck)*dnon_poiss;
    cv_log = cv/(nrec_bins(rk)*nrec_bins(ck));
    cov_logNK(rk,ck) = cv_log;
    cov_logNK(ck,rk) = cv_log;
end

     %------------
function M_NK = MLOC(specdat, eta,dAs)
N = trapz(specdat.egys,specdat.specnum);
Nrec_bar = (N/(1 + eta)); % mean recorded counts
sp_w_pileup = SpectrumWithPileup(specdat,eta);
[fracs,nbins] = BinFracsFromIndexes(sp_w_pileup, specdat.idx_threshold);
nrec_bins = Nrec_bar*fracs; % nbars for each bin at operating point
    % the M matrix
         % approach is filter input spectrum by additional dA_offset
         % compute average bin counts. 
         % Subtract log from initial and divide by dA_offset to give gradient   
M_NK = zeros(nbins,specdat.nbasis);
for kdim = 1:specdat.nbasis 
    sp2 = specdat;
    sp2.specnum = exp(-specdat.mus*dAs(kdim,:)').*specdat.specnum; % note transpose
    N2 = trapz(sp2.egys,sp2.specnum);
    eta2 = eta*(N2/N);
    sp_w_pileup2 = SpectrumWithPileup(sp2,eta2);
    fracs2 = BinFracsFromIndexes(sp_w_pileup2, specdat.idx_threshold);
    Nrec_bar2 = N2/(1 + eta2);
    nrec_bins2 = Nrec_bar2*fracs2; % nbars for each bin
    M_NK(:,kdim) = -(log(nrec_bins2) - log(nrec_bins))/dAs(kdim,kdim);
end

     %------------
function [fracs,nbins,idxs] = BinFracsFromIndexes(sp,idx_threshold)
% function fracs = BinFracsFromIndexes(sp,idx_threshold)
%  Compute fractions of photons in energy bins specified by idx_threshold into sp.specnum and sp.egys
%  inputs:
%      sp: a spectrum struct with fields
%          egys: the photon energies (keV)--assumed equally spaced
%          specnum: the number spectrum
%       indexes: a vector with threshold energies assumed to be monotonically increasing
%  outputs:
%      fracs: the fractions. Sum(fracs) = 1
%      Ebars: the average energy of each bin
%      idx_threshold: the indexes into sp.egys array corresponding to Ethresholds
%  REA 2011-Jul-24 10:03
%  COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

nbins = numel(idx_threshold) + 1;
bin_edges = [0; idx_threshold(:); numel(sp.egys)];
idxs = cell(nbins,1);
specnum = sp.specnum/sum(sp.specnum); % normalize it
fracs = zeros(nbins,1);
dE = sp.egys(2) - sp.egys(1);
for k = 1:nbins
    idxs{k}= (bin_edges(k)+1):bin_edges(k+1);
    fracs(k) = sum(specnum(idxs{k}))*dE;
end
