function [M,stats] = M_NQ_with_deadtime(specdat, Tintegrate, deadtime, varargin)
% function [M,stats] = M_NQ_with_deadtime(specdat, Tintegrate, deadtime, varargin)
% gradient of log(I) vs. A with deadtime. Approach is brute-force compute derivatives
% M(i,j) ~ (log(I_i(A_j)) - log(I_i(A_j + dA_j) ) )/dA_j
%  inputs:
%     specdat: spectrum struct -- spectrum transmitted through object. --Specnum integral= rate*Tintegrate
%     Tintegrate: (seconds) the integrate time for detector. 
%     deadtime: the deadtime in units compatible with count rate from specdat
%     dA: (optional = [0.01, 0.01]) the step offset for derivative calculation
%  outputs:
%     M: the effective mu matrix
%     stats: useful variables
%  REA 2012-Apr-22 10:42 - 2012-Apr-22 15:43
%  COPYRIGHT:   Aprend Technology, 2012. All rights reserved.

nreqargs = 3;
assert(nargin>=nreqargs);
dA = 0.001*[1; 1];

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'da';         dA=varargin{i+1};   assert(numel(dA)==2);  dA = dA(:); i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

    % extract rate from integral of spectrum and integrate time
N = trapz(specdat.egys,specdat.specnum);
rate = N/Tintegrate;

    % normalize the input spectrum
specdat.specnum_norm = specdat.specnum/N;
M = zeros(2); % will hold results
   % get measured transmitted flux and energy with no offset

N0rec = N/(1+rate*deadtime);
Q0 = trapz(specdat.egys,specdat.egys(:).*specdat.specnum(:));  

   % step off in each A coordinate and get difference
As_matrix = [dA(1) 0; 0 dA(2)];  % offset vectors arranged in a matrix
for kdim = 1:2  
    spec2 = exp(-specdat.mus*As_matrix(kdim,:)').*specdat.specnum_norm; % note transpose
    frac = trapz(specdat.egys,spec2); % fraction of incident spectrum after dA step
    N2rec = (frac*N)/(1+frac*rate*deadtime);
    M(1,kdim) = -(log(N2rec) - log(N0rec))/dA(kdim);
         % Q
    Q2 = N*trapz(specdat.egys,specdat.egys(:).*spec2(:)); % multiply by N to un-normalize spec2
    M(2,kdim) = -(log(Q2) - log(Q0))/dA(kdim);
end
if nargout>1
     stats.nbar_incident = N;
     stats.rate = rate;
     stats.eta = rate*deadtime;
     stats.nbar_rec = N0rec;
     stats.Q = Q0;
     stats.dAmatrix = As_matrix;
     stats.Ebar = trapz(specdat.egys,specdat.egys(:).*specdat.specnum_norm(:));
     stats.E2bar = trapz(specdat.egys,specdat.egys(:).^2.*specdat.specnum_norm(:));
     stats.F = stats.E2bar/(stats.Ebar)^2;
     eta1 = 1 + stats.eta;
     stats.RL = (1/N)*[1/eta1,1/eta1;1/eta1, stats.F] ; % covariance of log with deadtime
     stats.NQbar = [N N*stats.Ebar];
     stats.Lbar = log(stats.NQbar); % mean values of log vector
end     