function [idx_opt,Ethreshold,SNRmax,CRopt] = OptimNbinsThresholdsSNR(specdat,nbins,varargin)
% function [idx_opt,Ethreshold,SNRmax] = OptimNbinsThresholdsSNR(specdat,nbins,varargin)
%  Determine optimal thresholds for given spectrum
%  Adjusts thresholds to optimize SNR. SNR is computed as dA'*CRLB*dA
%  where CRLB is the Cramer-Rao lower bound for the covariance matrix
%  for an NK (K=nbins)  detector. the signal dA is set to [1;-1]'
%  inputs:
%     specdat: an array giving specnum or a spectrum structure with fields: specnum egys and zmus
%     nbins: the number of bins
%     dA: (two-length vector) (optional = [1; -1]). The delta A used to compute SNR
%     zA: (complex: optional = 0) additional attenuator (A_1 + 1i*A_2)
%     min_bin_width: (integer: optional = 3) minimum bin width for search
%     partial_search: (logical: optional = false) for nbins>2, clips off some of the idxs cases using an ad-hoc test
%     dynamic_range: (double: optional = 1e4)  ratio of max to min spectrum values allowed for clipzeros
%     close_dist: (double: optional = 0)  norm of difference of idx_threshold and indexes for close_egys to accept for test
%     close_egys: (double vector: optional = [])   inter-bin energies for closeness test
%     verbose: (double: optional = false) if present prints debug info
%  outputs:
%     idx_opt: an (nbins-1) integer array of the indexes into the specdat.egys array for the optimal thresholds
%     Ethreshold: (nbins-1) vector of corresponding energies.
%     SNRmax: the optimal SNR
%     CRopt: the Cramer-Rao lower bound for covariance for the optimal bins
%  REA 2/13/10 - 5/20/14

  % process the optional arguments
nreqargs = 2;
assert(nargin>=nreqargs);
assert( isstruct(specdat) );
assert(size(specdat.mus,2)==2);
if ~isfield(specdat,'zmus')
    specdat.zmus = zcomp(specdat.mus);
end
zA = 0;
dA = [1; -1]; % column vector
min_bin_width = 3;
partial_search = false;
dynamic_range = 1e4; % ratio of max to min spectrum values allowed
close_dist = 0;
close_egys = [];
verbose = false;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'partial_search';          partial_search = true;
     case 'verbose';          verbose = true;
     case 'za';         zA=varargin{i+1};    i=i+1;
     case 'da';         dA=varargin{i+1};  assert(numel(dA)==2); dA = dA(:); i=i+1;
     case 'min_bin_width'; min_bin_width = varargin{i+1};  i=i+1;
     case 'dynamic_range'; dynamic_range = varargin{i+1};  i=i+1;
     case 'close_dist'; close_dist = varargin{i+1}; assert(close_dist>0); i=i+1;
     case 'close_egys'; close_egys = varargin{i+1}; assert(numel(close_egys)==(nbins-1));assert(all(close_egys(:)>0)); i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end


   % computed transmitted spectrum through specified zA attenuator
specdat.specnum = specdat.specnum(:)'.*exp(-zcomp(zA)*zcomp(specdat.zmus)');
   % clip off zeros from spectrum--first to last that are greater than zero
speclip = specdat;
minspec = max(speclip.specnum)/dynamic_range;
idx_first_zero = find(abs(speclip.specnum)>minspec,1,'first');
idx_last_zero = find(abs(speclip.specnum)>minspec,1,'last');
if verbose
    fprintf('nstart %d 1stclip %d last_clip %d\n',numel(specdat.specnum), idx_first_zero, idx_last_zero);
end
speclip.specnum = speclip.specnum(idx_first_zero:idx_last_zero);
speclip.specnum = speclip.specnum(:);
speclip.egys = speclip.egys(idx_first_zero:idx_last_zero);
speclip.zmus = speclip.zmus(idx_first_zero:idx_last_zero);
speclip.mus = zcomp(speclip.zmus(:));
     
   % determine optimal partition by search through space of all possible indexes

% find all permutations of bin widths that sum to the number of energies
didxs = min_bin_width*(allVL1(nbins,floor(numel(speclip.egys)/min_bin_width) - nbins,'==') + 1);

   % exhaustively search them to find max SNR
n2search = size(didxs,1);
snrs_srch = zeros(n2search,1);
idxs = cumsum(didxs,2);
idxs = idxs(:,1:(end-1)); % last bin is from index to end
do_it = true(n2search,1);
if nbins>2 
    if partial_search
        negys = numel(speclip.egys);
        do_it = (didxs(:,1)>0.1*negys) &  (didxs(:,2)>0.1*negys) ;
        if verbose
            fprintf('limiting search range: searching %d out of %d\n',sum(do_it),n2search);
        end
    elseif close_dist>eps
        assert(~isempty(close_egys));
        idx_close = round(interp1(speclip.egys,1:numel(speclip.egys),close_egys));
        diffs = bsxfun(@minus,idxs,idx_close(:)'); % (:)' so have row vector
        do_it = sum(abs(diffs),2) < close_dist;
        if verbose
            fprintf('limiting search range by close test: searching %d out of %d\n',sum(do_it),n2search);
        end
    elseif verbose
        fprintf('n2search %d\n',n2search);
    end
end
for ks = 1:n2search
    if ~do_it(ks)
        continue;
    end
  snrs_srch(ks) = SNRloc( idxs(ks,:), speclip, dA);
end
[SNRmax,idx_max_SNR] = max(snrs_srch);
idx_opt = idxs(idx_max_SNR,:);
[tmp,CRopt] = SNRloc(idx_opt,speclip, dA);
assert(abs(tmp-SNRmax)<eps);
Ethreshold = speclip.egys(idx_opt);
idx_opt = idx_opt + idx_first_zero; % indexes for original unclipped spectrum

      %------ LOCAL FUNCTIONS -------------------
            function [snr,CR] = SNRloc(idx,sp_loc, dA)
                  CR = CRboundLoc(sp_loc,idx);
                  snr = dA'*inv(CR)*dA;                
                  
                
           function CR  = CRboundLoc(sp,idx_threshold)
              nbins = numel(idx_threshold)+1;       
              bin_edges = [0; idx_threshold(:);numel(sp.egys)];
%               idxs = cell(nbins,1);
%               for k = 1:nbins
%                   idxs{k}= (bin_edges(k)+1):bin_edges(k+1);
%               end
              M = zeros(nbins,2);
              lambdas = zeros(nbins,1);
              dE = sp.egys(2) - sp.egys(1);
              for kbin = 1:nbins
                  idx = (bin_edges(kbin)+1):bin_edges(kbin+1); %idxs{kbin};
                  spec_transmitted = sp.specnum(idx);
                  lambdas(kbin) = sum(spec_transmitted)*dE;
                  sp_norm = spec_transmitted(:)/ lambdas(kbin);
                     % compute M for counting signals
%                   mz = sum(sp.zmus(idx).*sp_norm)*dE;
%                   M(kbin,:) = zcomp(mz);
                      M(kbin,:) = sum(bsxfun(@times,sp.mus(idx,:),sp_norm));
              end
              RL = diag(1./lambdas);
              CR = inv(M'*inv(RL)*M);