function stats = SNR_with_pileup(specdat, eta, varargin)
%  function stats = SNR_with_pileup(specdat, eta, varargin)
%  SNR for ideal, N, Q, NQ detectors with pileup
%  inputs:
%     specdat: spectrum struct -- spectrum transmitted through object. before detector so no pileup
%        uses following fields:
%        specnum: photon number spectrum. The integral is the number of counts
%        egys: the corresponding energies (keV)
%        mus: matrix of samples of the basis set functions at egys. Each function is separate column
%     eta: the average number of photons per dead time 
%     dA: (optional = [1 -1]) object differences for SNR. should be multiplied by feature thickness but normalized by dividing by d2ideal
%     m_da_offset: (optional = [0.01, 0.01]) the step offset for derivative calculation
%     debug: (optional = false) if present does debugging tests (warning: outputs may have errors if this is used)
%  outputs: stats: a struct. see code for fields
%  REA 2012-Apr-23 15:21 - 2015-Feb-3
%  COPYRIGHT:   Aprend Technology, 2012-2105. All rights reserved.

nreqargs = 2;
assert(nargin>=nreqargs);
dA = [1; -1];
m_da_offset = 0.001*[1 1]; % offset to step off for M with deadtime calculation
debug = false;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
         case 'debug';   debug = true;
     case 'da';         dA=varargin{i+1};   assert(numel(dA)==2);  dA = dA(:); i=i+1;
     case 'm_da_offset';         m_da_offset=varargin{i+1};   assert(numel(m_da_offset)==2);  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

% detector_names = {'ideal','N','Q','NQ','PHA'};
   % spectra
nbar_incident = trapz(specdat.egys,specdat.specnum); % input counts per measurement time
specdat.specnum_norm = specdat.specnum/nbar_incident;
specdat.specegy = specdat.egys.*specdat.specnum; 
specdat.specegy_norm = specdat.specegy/trapz(specdat.egys,specdat.specegy);  
 
  % parameters for spectrum
Ebar = trapz(specdat.egys,specdat.specnum_norm.*specdat.egys);
E2bar = trapz(specdat.egys,specdat.specnum_norm.*(specdat.egys.^2));
F = E2bar/Ebar^2;

 % ideal SNR^2
dMus = diff(specdat.mus,1,2); % difference along rows
SNR_ideal = trapz(specdat.egys,specdat.specnum.*(dMus.^2));

    % SNR of Q (energy integrating) not affected by pileup
dmusQ = trapz(specdat.egys,specdat.specegy_norm.*dMus);
SNR_Q = nbar_incident*dmusQ^2/F;

     % SNR NQ detector
[RA_NQ, nqstats] = CRLB_NQ_with_pileup(specdat,eta);
SNR_NQ = dA'*nqstats.F*dA;

     % SNR N detector with pileup
M_N = nqstats.M(1,:); % eff mus for photon count spectrum with pileup
SNR_N = dA'*M_N'*(1/nqstats.RL(1,1))*M_N*dA; % nqstats.RL is cov of log of NrecQ
   % so nqstats.RL(1,1) is variance of log(Nrec)

   % load output struct with all variables in workspace.
stats = GetWorkspace;           